@@ -30,14 +30,11 @@ pub fn generate(
         .collect::<Vec<_>>();
     let dart_structs = distinct_types
         .iter()
-        .filter_map(|ty| {
-            if let StructRef(s) = ty {
-                Some(s.get(api_file))
-            } else {
-                None
-            }
+        .filter_map(|ty| match ty {
+            StructRef(s) => Some(generate_api_struct(s.get(api_file))),
+            EnumRef(enu) => Some(generate_api_enum(enu.get(api_file))),
+            _ => None,
         })
-        .map(generate_api_struct)
         .collect::<Vec<_>>();
     let dart_api2wire_funcs = distinct_input_types
         .iter()
@@ -52,6 +49,17 @@ pub fn generate(
         .map(|ty| generate_wire2api_func(ty, api_file))
         .collect::<Vec<_>>();
 
+    let needs_freezed = distinct_types
+        .iter()
+        .any(|ty| matches!(ty, EnumRef(e) if e.is_struct));
+    let freezed_header = if needs_freezed {
+        "import 'package:freezed_annotation/freezed_annotation.dart';
+        // import 'package:flutter/foundation.dart';
+        part 'bridge_generated.freezed.dart';"
+    } else {
+        ""
+    };
+
     let header = format!(
         "{}
 
@@ -64,7 +72,8 @@ pub fn generate(
     );
 
     let api_class = format!(
-        "abstract class {} extends FlutterRustBridgeBase<{}> {{
+        "{}
+        abstract class {} extends FlutterRustBridgeBase<{}> {{
             factory {}(ffi.DynamicLibrary dylib) => {}.raw({}(dylib));
 
             {}.raw({} inner) : super(inner);
@@ -76,6 +85,7 @@ pub fn generate(
 
         // ------------------------- Implementation Details -------------------------
         ",
+        freezed_header,
         dart_api_class_name,
         dart_wire_class_name,
         dart_api_class_name,
@@ -85,7 +95,7 @@ pub fn generate(
         dart_wire_class_name,
         dart_func_signatures_and_implementations
             .iter()
-            .map(|(sig, _, comm)| format!("{}\n{}", comm, sig))
+            .map(|(sig, _, comm)| format!("{}{}", comm, sig))
             .collect::<Vec<_>>()
             .join("\n\n"),
         dart_structs.join("\n\n"),
@@ -191,7 +201,7 @@ fn generate_api_func(func: &ApiFunc) -> (String, String, String) {
             debugName: \"{}\",
             argNames: [{}],
         ),
-        argValues: [{}], 
+        argValues: [{}],
         hint: hint,
         ",
         func.name,
@@ -302,8 +312,9 @@ fn generate_api2wire_func(ty: &ApiType) -> String {
                 )
             }
         },
-        // skip
-        StructRef(_) => return "".to_string(),
+        EnumRef(e) if !e.is_struct => "return raw.index;".to_owned(),
+        // skip, handled by transfomers
+        StructRef(_) | EnumRef(_) => return "".to_string(),
     };
 
     format!(
@@ -335,6 +346,62 @@ fn generate_api_fill_to_wire_func(ty: &ApiType, api_file: &ApiFile) -> String {
                 .collect::<Vec<_>>()
                 .join("\n")
         }
+        EnumRef(enu) if enu.is_struct => enu
+            .get(api_file)
+            .variants()
+            .iter()
+            .enumerate()
+            .map(|(idx, variant)| {
+                if let ApiVariantKind::Value = &variant.kind {
+                    format!(
+                        "if (apiObj is {}) {{ wireObj.tag = {}; return; }}",
+                        variant.name, idx
+                    )
+                } else {
+                    let r = format!("wireObj.kind.ref.{}.ref", variant.name);
+                    let body: Vec<_> = match &variant.kind {
+                        ApiVariantKind::Tuple(types) => types
+                            .iter()
+                            .enumerate()
+                            .map(|(idx, ty)| {
+                                format!(
+                                    "{}.field{1} = _api2wire_{2}(apiObj.field{1});",
+                                    r,
+                                    idx,
+                                    ty.safe_ident()
+                                )
+                            })
+                            .collect(),
+                        ApiVariantKind::Struct(st) => st
+                            .fields
+                            .iter()
+                            .map(|field| {
+                                format!(
+                                    "{}.{} = _api2wire_{}(apiObj.{});",
+                                    r,
+                                    field.name.rust_style(),
+                                    field.ty.safe_ident(),
+                                    field.name.dart_style()
+                                )
+                            })
+                            .collect(),
+                        _ => unreachable!(),
+                    };
+                    format!(
+                        "if (apiObj is {0}) {{
+                            wireObj.tag = {1};
+                            wireObj.kind = inner.inflate_{2}_{0}();
+                            {3}
+                        }}",
+                        variant.name,
+                        idx,
+                        enu.name,
+                        body.join("\n")
+                    )
+                }
+            })
+            .collect::<Vec<_>>()
+            .join("\n"),
         Optional(opt) => {
             if !opt.needs_initialization() || opt.is_list() {
                 return String::new();
@@ -348,7 +415,7 @@ fn generate_api_fill_to_wire_func(ty: &ApiType, api_file: &ApiFile) -> String {
             " _api_fill_to_wire_{}(apiObj, wireObj.ref);",
             boxed.inner.safe_ident()
         ),
-        Primitive(_) | Delegate(_) | PrimitiveList(_) | GeneralList(_) | Boxed(_) => {
+        Primitive(_) | Delegate(_) | PrimitiveList(_) | GeneralList(_) | Boxed(_) | EnumRef(_) => {
             return "".to_string();
         }
     };
@@ -415,6 +482,51 @@ fn generate_wire2api_func(ty: &ApiType, api_file: &ApiFile) -> String {
                 s.name, inner,
             )
         }
+        EnumRef(enu) if !enu.is_struct => format!("return {}.values[raw];", enu.name),
+        EnumRef(enu) => {
+            let enu = enu.get(api_file);
+            let variants = enu
+                .variants()
+                .iter()
+                .enumerate()
+                .map(|(idx, variant)| {
+                    let args = match &variant.kind {
+                        ApiVariantKind::Value => "()".to_owned(),
+                        ApiVariantKind::Tuple(types) => {
+                            let args = (1..=types.len())
+                                .map(|idx| format!("raw[{}]", idx))
+                                .collect::<Vec<_>>();
+                            format!("({})", args.join(","))
+                        }
+                        ApiVariantKind::Struct(s) => {
+                            let args = s
+                                .fields
+                                .iter()
+                                .enumerate()
+                                .map(|(idx, field)| {
+                                    format!("{}: raw[{}]", field.name.dart_style(), idx + 1)
+                                })
+                                .collect::<Vec<_>>();
+                            format!("({})", args.join(","))
+                        }
+                    };
+                    format!(
+                        "case {}: return {}.{}{};",
+                        idx,
+                        enu.name,
+                        variant.name.dart_style(),
+                        args
+                    )
+                })
+                .collect::<Vec<_>>();
+            format!(
+                "switch (raw[0]) {{
+                    {}
+                    default: throw Exception(\"unreachable\");
+                }}",
+                variants.join("\n"),
+            )
+        }
         Boxed(boxed) => match &boxed.inner {
             StructRef(inner) => format!("return _wire2api_{}(raw);", inner.safe_ident()),
             _ => gen_simple_type_cast(&ty.dart_api_type()),
@@ -432,12 +544,17 @@ fn generate_wire2api_func(ty: &ApiType, api_file: &ApiFile) -> String {
     )
 }
 
+/// A trailing newline is included if comments is not empty.
 fn dart_comments(comments: &[Comment]) -> String {
-    comments
+    let mut comments = comments
         .iter()
         .map(Comment::comment)
         .collect::<Vec<_>>()
-        .join("\n")
+        .join("\n");
+    if !comments.is_empty() {
+        comments.push('\n');
+    }
+    comments
 }
 
 fn generate_api_struct(s: &ApiStruct) -> String {
@@ -447,9 +564,8 @@ fn generate_api_struct(s: &ApiStruct) -> String {
         .map(|f| {
             let comments = dart_comments(&f.comments);
             format!(
-                "{}{}final {} {};",
+                "{}final {} {};",
                 comments,
-                if comments.is_empty() { "" } else { "\n" },
                 f.ty.dart_api_type(),
                 f.name.dart_style()
             )
@@ -467,8 +583,7 @@ fn generate_api_struct(s: &ApiStruct) -> String {
     let comments = dart_comments(&s.comments);
 
     format!(
-        "{}
-        class {} {{
+        "{}class {} {{
             {}
 
             {}({{{}}});
@@ -476,3 +591,83 @@ fn generate_api_struct(s: &ApiStruct) -> String {
         comments, s.name, field_declarations, s.name, constructor_params
     )
 }
+
+fn generate_api_enum(enu: &ApiEnum) -> String {
+    let comments = dart_comments(&enu.comments);
+    if enu.is_struct() {
+        let variants = enu
+            .variants()
+            .iter()
+            .map(|variant| {
+                let args = match &variant.kind {
+                    ApiVariantKind::Value => "".to_owned(),
+                    ApiVariantKind::Tuple(types) => {
+                        let split = optional_boundary_index(types);
+                        let types = types
+                            .iter()
+                            .enumerate()
+                            .map(|(idx, typ)| format!("{} field{}", typ.dart_api_type(), idx))
+                            .collect::<Vec<_>>();
+                        if let Some(idx) = split {
+                            let before = &types[..idx];
+                            let after = &types[idx..];
+                            let sep = if before.is_empty() { "" } else { "," };
+                            format!("{}{} [{}]", before.join(","), sep, after.join(","))
+                        } else {
+                            types.join(",")
+                        }
+                    }
+                    ApiVariantKind::Struct(st) => {
+                        let fields = st
+                            .fields
+                            .iter()
+                            .map(|field| {
+                                format!(
+                                    "{}{} {}",
+                                    field.ty.required_modifier(),
+                                    field.ty.dart_api_type(),
+                                    field.name.dart_style()
+                                )
+                            })
+                            .collect::<Vec<_>>();
+                        format!("{{ {} }}", fields.join(","))
+                    }
+                };
+                format!(
+                    "const factory {}.{}({}) = {};",
+                    enu.name,
+                    variant.name.dart_style(),
+                    args,
+                    variant.name.rust_style(),
+                )
+            })
+            .collect::<Vec<_>>();
+        format!(
+            "@freezed
+            class {0} with _${0} {{
+                {1}
+            }}",
+            enu.name,
+            variants.join("\n")
+        )
+    } else {
+        let variants = enu
+            .variants()
+            .iter()
+            .map(|variant| {
+                format!(
+                    "{}{},",
+                    dart_comments(&variant.comments),
+                    variant.name.rust_style()
+                )
+            })
+            .collect::<Vec<_>>()
+            .join("\n");
+        format!(
+            "{}enum {} {{
+            {}
+        }}",
+            comments, enu.name, variants
+        )
+    }
+}
