@@ -45,6 +45,13 @@ impl Generator {
             .iter()
             .map(|ty| self.generate_wire_struct(ty, api_file))
             .collect::<Vec<_>>();
+        let wire_enums = distinct_input_types
+            .iter()
+            .filter_map(|ty| match ty {
+                ApiType::EnumRef(enu) => Some(self.generate_wire_enum(enu, api_file)),
+                _ => None,
+            })
+            .collect::<Vec<_>>();
         let allocate_funcs = distinct_input_types
             .iter()
             .map(|f| self.generate_allocate_funcs(f))
@@ -63,7 +70,7 @@ impl Generator {
             .collect::<Vec<_>>();
 
         format!(
-            r#"#![allow(non_camel_case_types, unused, clippy::redundant_closure, clippy::useless_conversion)]
+            r#"#![allow(non_camel_case_types, unused, clippy::redundant_closure, clippy::useless_conversion, non_snake_case)]
         {}
 
         use crate::{}::*;
@@ -77,6 +84,10 @@ impl Generator {
 
         {}
 
+        // Section: wire enums
+
+        {}
+
         // Section: allocate functions
 
         {}
@@ -130,6 +141,7 @@ impl Generator {
             rust_wire_mod,
             wire_funcs.join("\n\n"),
             wire_structs.join("\n\n"),
+            wire_enums.join("\n\n"),
             allocate_funcs.join("\n\n"),
             wire2api_funcs.join("\n\n"),
             new_with_nullptr_funcs.join("\n\n"),
@@ -245,7 +257,7 @@ impl Generator {
                 .iter()
                 .map(std::ops::Deref::deref)
                 .collect::<Vec<_>>(),
-            return_type.as_deref(),
+            return_type,
             &format!(
                 "
                 {}.{}({}, move || {{
@@ -290,7 +302,9 @@ impl Generator {
                     })
                     .collect()
             }
-            Primitive(_) | Delegate(_) | Boxed(_) | Optional(_) => return "".to_string(),
+            Primitive(_) | Delegate(_) | Boxed(_) | Optional(_) | EnumRef(_) => {
+                return "".to_string()
+            }
         };
 
         format!(
@@ -306,6 +320,75 @@ impl Generator {
         )
     }
 
+    fn generate_wire_enum(&mut self, enu: &ApiTypeEnumRef, file: &ApiFile) -> String {
+        let src = enu.get(file);
+        if !src.is_struct() {
+            return "".to_owned();
+        }
+        let variant_structs = src
+            .variants()
+            .iter()
+            .map(|variant| {
+                let fields = match &variant.kind {
+                    ApiVariantKind::Value => vec![],
+                    ApiVariantKind::Tuple(types) => types
+                        .iter()
+                        .enumerate()
+                        .map(|(idx, typ)| {
+                            format!(
+                                "field{}: {}{},",
+                                idx,
+                                typ.rust_wire_modifier(),
+                                typ.rust_wire_type()
+                            )
+                        })
+                        .collect(),
+                    ApiVariantKind::Struct(s) => s
+                        .fields
+                        .iter()
+                        .map(|field| {
+                            format!(
+                                "{}: {}{},",
+                                field.name.rust_style(),
+                                field.ty.rust_wire_modifier(),
+                                field.ty.rust_wire_type()
+                            )
+                        })
+                        .collect(),
+                };
+                format!(
+                    "#[repr(C)]
+                    #[derive(Clone)]
+                    pub struct {}_{} {{ {} }}",
+                    enu.name,
+                    variant.name,
+                    fields.join("\n")
+                )
+            })
+            .collect::<Vec<_>>();
+        let union_fields = src
+            .variants()
+            .iter()
+            .map(|variant| format!("{0}: *mut {1}_{0},", variant.name, enu.name))
+            .collect::<Vec<_>>();
+        format!(
+            "#[repr(C)]
+            #[derive(Clone)]
+            pub struct {0} {{ tag: i32, kind: *mut {1}Kind }}
+
+            #[repr(C)]
+            pub union {1}Kind {{
+                {2}
+            }}
+
+            {3}",
+            enu.rust_wire_type(),
+            enu.name,
+            union_fields.join("\n"),
+            variant_structs.join("\n\n")
+        )
+    }
+
     fn generate_list_allocate_func(
         &mut self,
         safe_ident: &str,
@@ -370,7 +453,7 @@ impl Generator {
                     }
                 }
             },
-            Primitive(_) | Delegate(_) | StructRef(_) | Optional(_) => String::new(),
+            Primitive(_) | Delegate(_) | StructRef(_) | EnumRef(_) | Optional(_) => String::new(),
         }
     }
 
@@ -397,9 +480,13 @@ impl Generator {
             };
             vec.into_iter().map(Wire2Api::wire2api).collect()"
                 .into(),
-            Boxed(_) => "let wrap = unsafe { support::box_from_leak_ptr(self) };
-            (*wrap).wire2api().into()"
-                .into(),
+            Boxed(inner) => match inner.as_ref() {
+                ApiTypeBoxed { inner: ApiType::Primitive(_), exist_in_real_api: false } =>
+                    "unsafe { *support::box_from_leak_ptr(self) }".into(),
+                ApiTypeBoxed { inner: ApiType::Primitive(_), exist_in_real_api: true } =>
+                    "unsafe { support::box_from_leak_ptr(self) }".into(),
+                _ => "let wrap = unsafe { support::box_from_leak_ptr(self) }; (*wrap).wire2api().into()".into()
+            }
             StructRef(struct_ref) => {
                 let api_struct = struct_ref.get(api_file);
 
@@ -427,6 +514,62 @@ impl Generator {
                 }
                 .into()
             }
+            EnumRef(enu) if enu.is_struct => {
+                let enu = enu.get(api_file);
+                let variants = enu.variants().iter().enumerate()
+                    .map(|(idx, variant)| {
+                        if let ApiVariantKind::Value = variant.kind {
+                            format!("{} => {}::{},", idx, enu.name, variant.name)
+                        } else {
+                            let body = match &variant.kind {
+                                ApiVariantKind::Tuple(types) => {
+                                    let elms = (0..types.len()).map(|idx| format!("ans.field{}.wire2api()", idx))
+                                        .collect::<Vec<_>>();
+                                    format!("({})", elms.join(","))
+                                }
+                                ApiVariantKind::Struct(st) => {
+                                    let fields = st.fields
+                                        .iter().map(|field| format!("{0}: ans.{0}.wire2api()", field.name.rust_style()))
+                                        .collect::<Vec<_>>();
+                                    format!("{{ {} }}", fields.join(","))
+                                }
+                                _ => unreachable!()
+                            };
+                            format!(
+                                "{0} => unsafe {{
+                                    let ans = support::box_from_leak_ptr(self.kind);
+                                    let ans = support::box_from_leak_ptr(ans.{2});
+                                    {1}::{2}{3}
+                                }}",
+                                idx, enu.name, variant.name, body)
+                        }
+                    }).collect::<Vec<_>>();
+                format!(
+                    "match self.tag {{
+                        {}
+                        _ => unreachable!(),
+                    }}",
+                    variants.join("\n"),
+                ).into()
+            }
+            EnumRef(enu) => {
+                let enu = enu.get(api_file);
+                let variants = enu
+                    .variants()
+                    .iter()
+                    .enumerate()
+                    .map(|(idx, variant)| format!("{} => {}::{},", idx, enu.name, variant.name))
+                    .collect::<Vec<_>>()
+                    .join("\n");
+                format!(
+                    "match self {{
+                        {}
+                        _ => unreachable!(\"Invalid variant for {}: {{}}\", self),
+                    }}",
+                    variants, enu.name
+                )
+                .into()
+            }
             // handled by common impl
             Optional(_) => return String::new(),
         };
@@ -449,8 +592,11 @@ impl Generator {
         match ty {
             StructRef(st) => self
                 .generate_new_with_nullptr_func_for_struct(st.get(api_file), &ty.rust_wire_type()),
+            EnumRef(e) if e.is_struct => {
+                self.generate_new_with_nullptr_func_for_enum(e.get(api_file), &ty.rust_wire_type())
+            }
             Primitive(_) | Delegate(_) | PrimitiveList(_) | GeneralList(_) | Boxed(_)
-            | Optional(_) => String::new(),
+            | Optional(_) | EnumRef(_) => String::new(),
         }
     }
 
@@ -458,11 +604,78 @@ impl Generator {
         // println!("generate_impl_intodart: {:?}", ty);
         match ty {
             StructRef(s) => self.generate_impl_intodart_for_struct(s.get(api_file)),
+            EnumRef(e) if e.is_struct => {
+                self.generate_impl_intodart_for_enum_struct(e.get(api_file))
+            }
+            EnumRef(enu) => self.generate_impl_intodart_for_enum(enu.get(api_file)),
             Primitive(_) | Delegate(_) | PrimitiveList(_) | GeneralList(_) | Boxed(_)
             | Optional(_) => "".to_string(),
         }
     }
 
+    fn generate_new_with_nullptr_func_for_enum(
+        &mut self,
+        enu: &ApiEnum,
+        rust_wire_type: &str,
+    ) -> String {
+        fn init_of(ty: &ApiType) -> &str {
+            if ty.rust_wire_is_pointer() {
+                "core::ptr::null_mut()"
+            } else {
+                "Default::default()"
+            }
+        }
+        let inflators = enu
+            .variants()
+            .iter()
+            .filter_map(|variant| {
+                let typ = format!("{}_{}", enu.name, variant.name);
+                let body: Vec<_> = match &variant.kind {
+                    ApiVariantKind::Tuple(types) => types
+                        .iter()
+                        .enumerate()
+                        .map(|(idx, ty)| format!("field{}: {}", idx, init_of(ty)))
+                        .collect(),
+                    ApiVariantKind::Struct(st) => st
+                        .fields
+                        .iter()
+                        .map(|field| format!("{}: {}", field.name.rust_style(), init_of(&field.ty)))
+                        .collect(),
+                    _ => return None,
+                };
+                Some(self.extern_func_collector.generate(
+                    &format!("inflate_{}", typ),
+                    &[],
+                    Some(&format!("*mut {}Kind", enu.name)),
+                    &format!(
+                        "support::new_leak_box_ptr({}Kind {{
+                        {}: support::new_leak_box_ptr({} {{
+                            {}
+                        }})
+                    }})",
+                        enu.name,
+                        variant.name.rust_style(),
+                        typ,
+                        body.join(",")
+                    ),
+                ))
+            })
+            .collect::<Vec<_>>();
+        format!(
+            "impl NewWithNullPtr for {} {{
+                fn new_with_null_ptr() -> Self {{
+                    Self {{
+                        tag: -1,
+                        kind: core::ptr::null_mut(),
+                    }}
+                }}
+            }}
+            {}",
+            rust_wire_type,
+            inflators.join("\n\n")
+        )
+    }
+
     fn generate_new_with_nullptr_func_for_struct(
         &self,
         s: &ApiStruct,
@@ -476,7 +689,7 @@ impl Generator {
                         "{}: {},",
                         field.name.rust_style(),
                         if field.ty.rust_wire_is_pointer() {
-                            "std::ptr::null_mut()"
+                            "core::ptr::null_mut()"
                         } else {
                             "Default::default()"
                         }
@@ -522,6 +735,88 @@ impl Generator {
             s.name, body, s.name,
         )
     }
+
+    fn generate_impl_intodart_for_enum(&mut self, enu: &ApiEnum) -> String {
+        let variants = enu
+            .variants()
+            .iter()
+            .enumerate()
+            .map(|(idx, variant)| format!("Self::{} => {},", variant.name, idx))
+            .collect::<Vec<_>>()
+            .join("\n");
+        format!(
+            "impl support::IntoDart for {} {{
+                fn into_dart(self) -> support::DartCObject {{
+                    match self {{
+                        {}
+                    }}.into_dart()
+                }}
+            }}",
+            enu.name, variants
+        )
+    }
+
+    fn generate_impl_intodart_for_enum_struct(&mut self, enu: &ApiEnum) -> String {
+        let variants =
+            enu.variants()
+                .iter()
+                .enumerate()
+                .map(|(idx, variant)| {
+                    let tag = format!("{}.into_dart()", idx);
+                    match &variant.kind {
+                        ApiVariantKind::Value => {
+                            format!("Self::{} => vec![{}],", variant.name, tag)
+                        }
+                        ApiVariantKind::Tuple(types) => {
+                            let len = types.len();
+                            let fields = Some(tag)
+                                .into_iter()
+                                .chain((0..len).map(|idx| format!("field{}.into_dart()", idx)))
+                                .collect::<Vec<_>>();
+                            let pattern = (0..len)
+                                .map(|idx| format!("field{}", idx))
+                                .collect::<Vec<_>>();
+                            format!(
+                                "Self::{}({}) => vec![{}],",
+                                variant.name,
+                                pattern.join(","),
+                                fields.join(",")
+                            )
+                        }
+                        ApiVariantKind::Struct(s) => {
+                            let fields = Some(tag)
+                                .into_iter()
+                                .chain(s.fields.iter().map(|field| {
+                                    format!("{}.into_dart()", field.name.rust_style())
+                                }))
+                                .collect::<Vec<_>>();
+                            let pattern = s
+                                .fields
+                                .iter()
+                                .map(|field| field.name.rust_style().to_owned())
+                                .collect::<Vec<_>>();
+                            format!(
+                                "Self::{}{{ {} }} => vec![{}],",
+                                variant.name,
+                                pattern.join(","),
+                                fields.join(",")
+                            )
+                        }
+                    }
+                })
+                .collect::<Vec<_>>();
+        format!(
+            "impl support::IntoDart for {} {{
+                fn into_dart(self) -> support::DartCObject {{
+                    match self {{
+                        {}
+                    }}.into_dart()
+                }}
+            }}",
+            enu.name,
+            variants.join("\n")
+        )
+    }
 }
 
 struct ExternFuncCollector {
