@@ -13,32 +13,47 @@ use crate::api_types::*;
 use crate::generator_rust::HANDLER_NAME;
 
 type StructMap<'a> = HashMap<String, &'a ItemStruct>;
+type EnumMap<'a> = HashMap<String, &'a ItemEnum>;
 
 pub fn parse(source_rust_content: &str, file: File) -> ApiFile {
-    let (src_fns, src_struct_map) = extract_items_from_file(&file);
+    let SrcItems {
+        src_fns,
+        src_struct_map,
+        src_enums,
+    } = extract_items_from_file(&file);
     let parser = Parser {
         src_struct_map,
+        src_enums,
         struct_pool: HashMap::new(),
+        enum_pool: HashMap::new(),
         parsing_or_parsed_struct_names: HashSet::new(),
+        parsed_enums: HashSet::new(),
     };
     parser.parse(source_rust_content, src_fns)
 }
 
 struct Parser<'a> {
     src_struct_map: HashMap<String, &'a ItemStruct>,
-    struct_pool: ApiStructPool,
     parsing_or_parsed_struct_names: HashSet<String>,
+    struct_pool: ApiStructPool,
+
+    src_enums: HashMap<String, &'a ItemEnum>,
+    parsed_enums: HashSet<String>,
+    enum_pool: ApiEnumPool,
 }
 
-fn extract_comments(attr: &Attribute) -> Option<Comment> {
-    match attr.parse_meta() {
-        Ok(Meta::NameValue(MetaNameValue {
-            path,
-            lit: Lit::Str(lit),
-            ..
-        })) if path.is_ident("doc") => Some(Comment::from(lit.value().as_ref())),
-        _ => None,
-    }
+fn extract_comments(attrs: &[Attribute]) -> Vec<Comment> {
+    attrs
+        .iter()
+        .filter_map(|attr| match attr.parse_meta() {
+            Ok(Meta::NameValue(MetaNameValue {
+                path,
+                lit: Lit::Str(lit),
+                ..
+            })) if path.is_ident("doc") => Some(Comment::from(lit.value().as_ref())),
+            _ => None,
+        })
+        .collect()
 }
 
 impl<'a> Parser<'a> {
@@ -50,6 +65,7 @@ impl<'a> Parser<'a> {
         ApiFile {
             funcs,
             struct_pool: self.struct_pool,
+            enum_pool: self.enum_pool,
             has_executor,
         }
     }
@@ -62,7 +78,7 @@ impl<'a> Parser<'a> {
         }
 
         let sig = &func.sig;
-        let func_name = ident_to_string(&sig.ident);
+        let func_name = sig.ident.to_string();
 
         let mut inputs = Vec::new();
         let mut output = None;
@@ -81,11 +97,10 @@ impl<'a> Parser<'a> {
                     output = Some(stream_sink_inner_type);
                     mode = Some(ApiFuncMode::Stream);
                 } else {
-                    let comments = pat_type.attrs.iter().filter_map(extract_comments).collect();
                     inputs.push(ApiField {
                         name: ApiIdent::new(name),
                         ty: self.parse_type(&type_string),
-                        comments,
+                        comments: extract_comments(&pat_type.attrs),
                     });
                 }
             } else {
@@ -113,14 +128,14 @@ impl<'a> Parser<'a> {
             );
         }
 
-        let comments = func.attrs.iter().filter_map(extract_comments).collect();
+        // let comments = func.attrs.iter().filter_map(extract_comments).collect();
 
         ApiFunc {
             name: func_name,
             inputs,
             output: output.expect("unsupported output"),
             mode: mode.expect("unsupported mode"),
-            comments,
+            comments: extract_comments(&func.attrs),
         }
     }
 
@@ -132,6 +147,7 @@ impl<'a> Parser<'a> {
             .or_else(|| self.try_parse_box(ty))
             .or_else(|| self.try_parse_option(ty))
             .or_else(|| self.try_parse_struct(ty))
+            .or_else(|| self.try_parse_enum(ty))
             .unwrap_or_else(|| panic!("parse_type failed for ty={}", ty))
     }
 
@@ -241,6 +257,71 @@ impl<'a> Parser<'a> {
         }))
     }
 
+    fn try_parse_enum(&mut self, ty: &str) -> Option<ApiType> {
+        if !self.src_enums.contains_key(ty) {
+            return None;
+        }
+
+        if self.parsed_enums.insert(ty.to_owned()) {
+            let enu = self.parse_enum(ty);
+            self.enum_pool.insert(ty.to_owned(), enu);
+        }
+
+        Some(EnumRef(ApiTypeEnumRef {
+            name: ty.to_owned(),
+            is_struct: self
+                .enum_pool
+                .get(ty)
+                .map(ApiEnum::is_struct)
+                .unwrap_or(true),
+        }))
+    }
+
+    fn parse_enum(&mut self, ty: &str) -> ApiEnum {
+        let src = self.src_enums[ty];
+        let name = src.ident.to_string();
+        let comments = extract_comments(&src.attrs);
+        let variants = src
+            .variants
+            .iter()
+            .map(|variant| ApiVariant {
+                name: ApiIdent::new(variant.ident.to_string()),
+                comments: extract_comments(&variant.attrs),
+                kind: match variant.fields.iter().next() {
+                    None => ApiVariantKind::Value,
+                    Some(Field { ident: None, .. }) => ApiVariantKind::Tuple(
+                        variant
+                            .fields
+                            .iter()
+                            .map(|field| self.parse_type(&type_to_string(&field.ty)))
+                            .collect(),
+                    ),
+                    Some(Field {
+                        ident: Some(_),
+                        attrs,
+                        ..
+                    }) => ApiVariantKind::Struct(ApiStruct {
+                        name: variant.ident.to_string(),
+                        is_fields_named: true,
+                        comments: extract_comments(attrs),
+                        fields: variant
+                            .fields
+                            .iter()
+                            .map(|field| ApiField {
+                                name: ApiIdent::new(
+                                    field.ident.as_ref().map(ToString::to_string).unwrap(),
+                                ),
+                                ty: self.parse_type(&type_to_string(&field.ty)),
+                                comments: extract_comments(&field.attrs),
+                            })
+                            .collect(),
+                    }),
+                },
+            })
+            .collect();
+        ApiEnum::new(name, comments, variants)
+    }
+
     fn parse_struct_core(&mut self, ty: &str) -> ApiStruct {
         let item_struct = self.src_struct_map[ty];
         let mut fields = Vec::new();
@@ -255,23 +336,18 @@ impl<'a> Parser<'a> {
             let field_name = field
                 .ident
                 .as_ref()
-                .map_or(format!("field{}", idx), |id| ident_to_string(id));
+                .map_or(format!("field{}", idx), ToString::to_string);
             let field_type_str = type_to_string(&field.ty);
             let field_type = self.parse_type(&field_type_str);
-            let comments = field.attrs.iter().filter_map(extract_comments).collect();
             fields.push(ApiField {
                 name: ApiIdent::new(field_name),
                 ty: field_type,
-                comments,
+                comments: extract_comments(&field.attrs),
             });
         }
 
-        let name = ident_to_string(&item_struct.ident);
-        let comments = item_struct
-            .attrs
-            .iter()
-            .filter_map(extract_comments)
-            .collect();
+        let name = item_struct.ident.to_string();
+        let comments = extract_comments(&item_struct.attrs);
         ApiStruct {
             name,
             fields,
@@ -281,9 +357,16 @@ impl<'a> Parser<'a> {
     }
 }
 
-fn extract_items_from_file(file: &File) -> (Vec<&ItemFn>, StructMap) {
+struct SrcItems<'a> {
+    src_fns: Vec<&'a ItemFn>,
+    src_struct_map: StructMap<'a>,
+    src_enums: EnumMap<'a>,
+}
+
+fn extract_items_from_file(file: &File) -> SrcItems {
     let mut src_fns = Vec::new();
     let mut src_struct_map = HashMap::new();
+    let mut src_enums = HashMap::new();
     for item in file.items.iter() {
         match item {
             Item::Fn(ref item_fn) => {
@@ -296,21 +379,31 @@ fn extract_items_from_file(file: &File) -> (Vec<&ItemFn>, StructMap) {
                     src_struct_map.insert(item_struct.ident.to_string(), item_struct);
                 }
             }
+            Item::Enum(
+                item_enu
+                @
+                ItemEnum {
+                    vis: Visibility::Public(_),
+                    ..
+                },
+            ) => {
+                src_enums.insert(item_enu.ident.to_string(), item_enu);
+            }
             _ => {}
         }
     }
     // println!("[Functions]\n{:#?}", src_fns);
     // println!("[Structs]\n{:#?}", src_struct_map);
-    (src_fns, src_struct_map)
-}
-
-fn ident_to_string(ident: &Ident) -> String {
-    format!("{}", ident)
+    SrcItems {
+        src_fns,
+        src_struct_map,
+        src_enums,
+    }
 }
 
 /// syn -> string https://github.com/dtolnay/syn/issues/294
 fn type_to_string(ty: &Type) -> String {
-    quote!(#ty).to_string().replace(" ", "")
+    quote!(#ty).to_string().replace(' ', "")
 }
 
 struct GenericCapture {
@@ -319,7 +412,8 @@ struct GenericCapture {
 
 impl GenericCapture {
     pub fn new(cls_name: &str) -> Self {
-        let regex = Regex::new(&*format!("^[^<]*{}<([a-zA-Z0-9_<>]+|\\(\\))>$", cls_name)).unwrap();
+        let regex =
+            Regex::new(&*format!("^[^<]*{}<([a-zA-Z0-9_<>()\\[\\];]+)>$", cls_name)).unwrap();
         Self { regex }
     }
 
